@@ -24,8 +24,10 @@
 #include "config.h"
 #endif
 
-#if !defined(__STDC__) && !defined(const)
-#define const
+#ifndef __STDC__
+#  ifndef const
+#    define const
+#  endif
 #endif
 
 /* This tells Alpha OSF/1 not to define a getopt prototype in <stdio.h>.  */
@@ -117,7 +119,8 @@ int opterr = 1;
    This must be initialized on some systems to avoid linking in the
    system's own getopt implementation.  */
 
-int optopt = '?';
+#define BAD_OPTION '\0'
+int optopt = BAD_OPTION;
 
 /* Describe how to deal with options that follow non-option ARGV-elements.
 
@@ -244,22 +247,24 @@ exchange (argv)
   /* Reverse all the elements [first_nonopt, optind) */
   first = &argv[first_nonopt];
   last  = &argv[optind-1];
-  while (first < last)
-    temp = *first, *first = *last, *last = temp, first++, last--;
-
+  while (first < last) {
+    temp = *first; *first = *last; *last = temp; first++; last--;
+  }
   /* Put back the options in order */
   first = &argv[first_nonopt];
   first_nonopt += (optind - last_nonopt);
   last  = &argv[first_nonopt - 1];
-  while (first < last)
-    temp = *first, *first = *last, *last = temp, first++, last--;
+  while (first < last) {
+    temp = *first; *first = *last; *last = temp; first++; last--;
+  }
 
   /* Put back the non options in order */
   first = &argv[first_nonopt];
   last_nonopt = optind;
   last  = &argv[last_nonopt-1];
-  while (first < last)
-    temp = *first, *first = *last, *last = temp, first++, last--;
+  while (first < last) {
+    temp = *first; *first = *last; *last = temp; first++; last--;
+  }
 }
 
 /* Scan elements of ARGV (whose length is ARGC) for option characters
@@ -282,8 +287,8 @@ exchange (argv)
 
    OPTSTRING is a string containing the legitimate option characters.
    If an option character is seen that is not listed in OPTSTRING,
-   return '?' after printing an error message.  If you set `opterr' to
-   zero, the error message is suppressed but we still return '?'.
+   return BAD_OPTION after printing an error message.  If you set `opterr' to
+   zero, the error message is suppressed but we still return BAD_OPTION.
 
    If a char in OPTSTRING is followed by a colon, that means it wants an arg,
    so the following text in the same ARGV-element, or the text of the following
@@ -488,7 +493,7 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
 		     argv[0], argv[optind]);
 	  nextchar += my_strlen (nextchar);
 	  optind++;
-	  return '?';
+	  return BAD_OPTION;
 	}
 
       if (pfound != NULL)
@@ -517,7 +522,7 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
 			     argv[0], argv[optind - 1][0], pfound->name);
 		    }
 		  nextchar += my_strlen (nextchar);
-		  return '?';
+		  return BAD_OPTION;
 		}
 	    }
 	  else if (pfound->has_arg == 1)
@@ -530,7 +535,7 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
 		    fprintf (stderr, "%s: option `%s' requires an argument\n",
 			     argv[0], argv[optind - 1]);
 		  nextchar += my_strlen (nextchar);
-		  return optstring[0] == ':' ? ':' : '?';
+		  return optstring[0] == ':' ? ':' : BAD_OPTION;
 		}
 	    }
 	  nextchar += my_strlen (nextchar);
@@ -566,7 +571,7 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
 	    }
 	  nextchar = (char *) "";
 	  optind++;
-	  return '?';
+	  return BAD_OPTION;
 	}
     }
 
@@ -596,7 +601,7 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
 #endif
 	  }
 	optopt = c;
-	return '?';
+	return BAD_OPTION;
       }
     if (temp[1] == ':')
       {
@@ -639,7 +644,7 @@ _getopt_internal (argc, argv, optstring, longopts, longind, long_only)
 		if (optstring[0] == ':')
 		  c = ':';
 		else
-		  c = '?';
+		  c = BAD_OPTION;
 	      }
 	    else
 	      /* We already incremented `optind' once;
@@ -728,7 +733,7 @@ main (argc, argv)
 	  printf ("option c with value `%s'\n", optarg);
 	  break;
 
-	case '?':
+	case BAD_OPTION:
 	  break;
 
 	default:
